@file:OptIn(ExperimentalMaterial3Api::class)

package com.example.app.screens

import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import com.arcgismaps.Color
import com.arcgismaps.data.ServiceFeatureTable
import com.arcgismaps.mapping.ArcGISMap
import com.arcgismaps.mapping.BasemapStyle
import com.arcgismaps.mapping.Viewpoint
import com.arcgismaps.mapping.labeling.ArcadeLabelExpression
import com.arcgismaps.mapping.labeling.LabelDefinition
import com.arcgismaps.mapping.layers.FeatureLayer
import com.arcgismaps.mapping.symbology.ClassBreak
import com.arcgismaps.mapping.symbology.ClassBreaksRenderer
import com.arcgismaps.mapping.symbology.FontStyle
import com.arcgismaps.mapping.symbology.FontWeight
import com.arcgismaps.mapping.symbology.PictureMarkerSymbol
import com.arcgismaps.mapping.symbology.SimpleFillSymbol
import com.arcgismaps.mapping.symbology.SimpleFillSymbolStyle
import com.arcgismaps.mapping.symbology.SimpleLineSymbol
import com.arcgismaps.mapping.symbology.SimpleLineSymbolStyle
import com.arcgismaps.mapping.symbology.SimpleRenderer
import com.arcgismaps.mapping.symbology.TextSymbol
import com.arcgismaps.mapping.symbology.UniqueValue
import com.arcgismaps.mapping.symbology.UniqueValueRenderer
import com.arcgismaps.toolkit.geoviewcompose.MapView
import com.example.app.R

@Composable
fun MainScreen() {
    val map = remember {
        createMap()
    }
    Scaffold(
        topBar = { TopAppBar(title = { Text(text = stringResource(id = R.string.app_name)) }) }
    ) {
        MapView(
            modifier = Modifier.fillMaxSize().padding(it),
            arcGISMap = map
        )
    }
}

fun createMap(): ArcGISMap {

    val parksAndOpenSpaces =
        "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Parks_and_Open_Space/FeatureServer/0"
    val trails =
        "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trails/FeatureServer/0"
    val trailheads =
        "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trailheads/FeatureServer/0"
    val trailheadImage =
        "https://static.arcgis.com/images/Symbols/NPS/npsPictograph_0231b.png"

    return ArcGISMap(BasemapStyle.ArcGISTopographic).apply {
        initialViewpoint = Viewpoint(
            latitude = 34.0270,
            longitude = -118.8050,
            scale = 72000.0
        )

        operationalLayers.addAll(

            listOf(

                createOpenSpaceLayer(parksAndOpenSpaces),

                createTrailsLayer(trails),

                createBikeOnlyTrailsLayer(trails),

                createNoBikeTrailsLayer(trails),

                createTrailheadsLayer(trailheads, trailheadImage)

            )

        )

    }
}

fun createFeatureLayer(featureServiceUri: String): FeatureLayer {
    // Create a service feature table from a Uri
    val serviceFeatureTable = ServiceFeatureTable(featureServiceUri)
    // Return a feature layer created from the service feature table
    return FeatureLayer.createWithFeatureTable(serviceFeatureTable)
}

fun createOpenSpaceLayer(featureServiceUri: String): FeatureLayer {
    // Create fill symbol objects to represent the parks and open spaces layer
    val magentaFillSymbol = SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.fromRgba(255, 0, 255), null)
    val greenFillSymbol = SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.green, null)
    val blueFillSymbol = SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.fromRgba(0, 0, 255), null)
    val redFillSymbol = SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.red, null)

    // Create a unique value for natural areas, regional open spaces, local parks, and regional recreation parks
    val naturalAreas = UniqueValue(
        description = "Natural Areas",
        label = "Natural Areas",
        symbol = magentaFillSymbol,
        values = listOf("Natural Areas")
    )
    val regionalOpenSpace = UniqueValue(
        description = "Regional Open Space",
        label = "Regional Open Space",
        symbol = greenFillSymbol,
        values = listOf("Regional Open Space")
    )
    val localPark = UniqueValue(
        description = "Local Park",
        label = "Local Park",
        symbol = blueFillSymbol,
        values = listOf("Local Park")
    )
    val regionalRecreationPark = UniqueValue(
        description = "Regional Recreation Park",
        label = "Regional Recreation Park",
        symbol = redFillSymbol,
        values = listOf("Regional Recreation Park")
    )
    // Create a unique value list with the fill symbols
    val uniqueValuesList = listOf(
        naturalAreas,
        regionalOpenSpace,
        localPark,
        regionalRecreationPark
    )

    // Create and assign a unique value renderer to the feature layer
    val openSpacesUniqueValueRenderer =
        UniqueValueRenderer(
            fieldNames = listOf("TYPE"),
            uniqueValues = uniqueValuesList,
            defaultLabel = "Open Spaces",
            defaultSymbol = null
        )

    // Create a parks and open spaces feature layer
    val featureLayer = createFeatureLayer(featureServiceUri).apply {
        // Style the feature layer using the unique value renderer
        renderer = openSpacesUniqueValueRenderer
        // Set the layer opacity to semi-transparent
        opacity = 0.2f
    }

    return featureLayer
}

fun createTrailsLayer(featureServiceUri: String): FeatureLayer {
    // Create simple symbol objects to represent the trails layer
    val firstClassSymbol = SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.fromRgba(255, 0, 255), 3.0f)
    val secondClassSymbol = SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.fromRgba(255, 0, 255), 4.0f)
    val thirdClassSymbol = SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.fromRgba(255, 0, 255), 5.0f)
    val fourthClassSymbol = SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.fromRgba(255, 0, 255), 6.0f)
    val fifthClassSymbol = SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.fromRgba(255, 0, 255), 7.0f)

    // Create 5 class breaks
    val firstClassBreak = ClassBreak(
        description = "Under 500",
        label = "0 - 500",
        minValue = 0.0,
        maxValue = 500.0,
        symbol = firstClassSymbol
    )
    val secondClassBreak = ClassBreak(
        description = "501 to 1000",
        label = "501 - 1000",
        minValue = 501.0,
        maxValue = 1000.0,
        symbol = secondClassSymbol
    )
    val thirdClassBreak = ClassBreak(
        description = "1001 to 1500",
        label = "1001 - 1500",
        minValue = 1001.0,
        maxValue = 1500.0,
        symbol = thirdClassSymbol
    )
    val fourthClassBreak = ClassBreak(
        description = "1501 to 2000",
        label = "1501 - 2000",
        minValue = 1501.0,
        maxValue = 2000.0,
        symbol = fourthClassSymbol
    )
    val fifthClassBreak = ClassBreak(
        description = "2001 to 2300",
        label = "2001 - 2300",
        minValue = 2001.0,
        maxValue = 2300.0,
        symbol = fifthClassSymbol
    )
    val elevationBreaks = listOf(
        firstClassBreak,
        secondClassBreak,
        thirdClassBreak,
        fourthClassBreak,
        fifthClassBreak
    )

    // Create and assign a class breaks renderer to the feature layer
    val elevationClassBreaksRenderer = ClassBreaksRenderer("ELEV_GAIN", elevationBreaks)

    // Create a trails feature layer
    val featureLayer = createFeatureLayer(featureServiceUri).apply {
        // Style the feature layer using the class breaks renderer
        renderer = elevationClassBreaksRenderer
        // Set the layer opacity to semi-transparent
        opacity = 0.75f
    }

    return featureLayer
}

fun createBikeOnlyTrailsLayer(featureServiceUri: String): FeatureLayer {
    // Create blue dot style simple line symbol to represent the bike trails
    val bikeTrailSymbol = SimpleLineSymbol(SimpleLineSymbolStyle.Dot, Color.fromRgba(0, 0, 255), 2.0f)
    // Create a simple renderer for the feature layer
    val bikeTrailRenderer = SimpleRenderer(bikeTrailSymbol)

    // Create a bike trails feature layer
    val featureLayer = createFeatureLayer(featureServiceUri).apply {
        // Style the feature layer using the simple renderer
        renderer = bikeTrailRenderer
        // Write a definition expression to filter for trails that permit the use of bikes
        definitionExpression = "USE_BIKE = 'Yes'"
    }

    return featureLayer
}

fun createNoBikeTrailsLayer(featureServiceUri: String): FeatureLayer {
    // Create a yellow dot style simple line symbol to represent no bike trails
    val noBikeTrailSymbol = SimpleLineSymbol(SimpleLineSymbolStyle.Dot, Color.fromRgba(255, 255, 0), 2.0f)
    // Create a simple renderer for the feature layer
    val noBikeTrailRenderer = SimpleRenderer(noBikeTrailSymbol)

    // Create a no-bike trails feature layer
    val featureLayer = createFeatureLayer(featureServiceUri).apply {
        // Style the feature layer using the simple renderer
        renderer = noBikeTrailRenderer
        // Write a definition expression to filter for trails that don't permit the use of bikes
        definitionExpression = "USE_BIKE = 'No'"
    }

    return featureLayer
}

fun createTrailheadsLayer(featureServiceUri: String, trailheadImage: String): FeatureLayer {
    // Create a new picture marker symbol that uses the trailhead image
    val pictureMarkerSymbol = PictureMarkerSymbol(trailheadImage).apply {
        height = 18.0f
        width = 18.0f
    }
    // Create a new simple renderer based on the picture marker symbol
    val simpleRenderer = SimpleRenderer(pictureMarkerSymbol)
    // Create the label definition
    val trailHeadsDefinition = makeLabelDefinition(labelAttribute = "TRL_NAME")

    // Create a trail heads feature layer
    val featureLayer: FeatureLayer = createFeatureLayer(featureServiceUri).apply {
        // Style the feature layer using the simple renderer
        renderer = simpleRenderer
        // Set labeling on the layer to be enabled
        labelsEnabled = true
        // Add the label definition to the layer's label definition collection
        labelDefinitions.add(trailHeadsDefinition)
    }

    return featureLayer
}

fun makeLabelDefinition(labelAttribute: String): LabelDefinition {
    // Create a text symbol for the label definition
    val labelTextSymbol = TextSymbol().apply {
        color = Color.white
        size = 12.0f
        haloColor = Color.red
        haloWidth = 1.0f
        fontFamily = "Arial"
        fontStyle = FontStyle.Italic
        fontWeight = FontWeight.Normal
    }

    // Create an Arcade label expression based on the field name
    val labelExpression = ArcadeLabelExpression("\$feature.$labelAttribute")

    // Create and return the label definition
    return LabelDefinition(labelExpression, labelTextSymbol)
}


